/*******************************************************************************
* File Name: main.c
*
* Version: 1.0
*
* Description:
*  This is source code for the data sheet example of the Real Time Clock (RTC)
*  component.
*
*  Display legend:
*   First string:
*    RTC HH:MM:SS AP
*       RTC- component's name
*       HH - hours
*       MM - minutes
*       SS - seconds
*       AP - AM/PM
*
*   Second string:
*    YY:MM:DD DW L D
*       YY - two last digits of the year
*       MM - month
*       DD - day of month
*       DW - day of week
*       L - present if year is leap year
*       D - DST is active
*
* Related Document:
*
* Hardware Dependency:
*  1. CY8C38 Family Processor Module rev4 with CY8C3866AXI-040 ES2/ES3
*  2. DVK CY8CKIT-001 Rev 4
*  3. Miniprog Rev *A
*
* Code Tested With:
*  1. PSoC Creator 1.0 Beta 5
*  2. DP8051-Keil Generic Compiler
*******************************************************************************/

#include <device.h>
#include "utils.h"


/*******************************************************************************
* Function Name: main
********************************************************************************
* Summary:
*  Main function performs following functions:
*   1. Declares date and time structure and fill it with the initial values
*   2. Enables global interrupts
*   3. Sets initial time and date values
*   4. Sets alarm date, time and mask
*   5. Sets interval mask
*   6. Sets DST start and stop configuration
*   7. Prints current date and time values.
*
*   The information on the display is updated in the ISR.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void main()
{
    uint8 tmpVar = 0u;

    RTC_TIME_DATE Start;

    /* Fill struct with date and time */
    Start.Sec = 55u;
    Start.Min = 59u;
    Start.Hour = 22u;
    Start.DayOfMonth = 31u;
    Start.Month = 12u;
    Start.Year = 2007u;

    /* Enable all interrupts */
    CYGlobalIntEnable;

    /* Set date and time */
    RTC_WriteTime(&Start);

    /* Set alarm date and time */
    RTC_WriteAlarmSecond(5u);
    RTC_WriteAlarmMinute(3u);
    RTC_WriteAlarmHour(1u);
    RTC_WriteAlarmDayOfMonth(1u);
    RTC_WriteAlarmMonth(1u);
    RTC_WriteAlarmYear(2008u);

    /* Set alarm mask */
    RTC_WriteAlarmMask(RTC_ALARM_SEC_MASK | RTC_ALARM_MIN_MASK |
                       RTC_ALARM_HOUR_MASK | RTC_ALARM_DAYOFMONTH_MASK |
                       RTC_ALARM_MONTH_MASK | RTC_ALARM_YEAR_MASK);

    /* Set interval mask - handling of interrupt stubs of the RTC component */
    RTC_WriteIntervalMask(RTC_INTERVAL_SEC_MASK | RTC_INTERVAL_MIN_MASK |
                          RTC_INTERVAL_HOUR_MASK | RTC_INTERVAL_DAY_MASK |
                          RTC_INTERVAL_WEEK_MASK | RTC_INTERVAL_MONTH_MASK |
                          RTC_INTERVAL_YEAR_MASK);

    /* DST start configuration */
    RTC_WriteDSTMode(RTC_DST_ENABLE | RTC_DST_FIXDATE);
    RTC_WriteDSTStartHour(23u);
    RTC_WriteDSTStartDayOfMonth(31u);
    RTC_WriteDSTStartMonth(12u);

    /* DST stop configuration */
    RTC_WriteDSTStopHour(2u);
    RTC_WriteDSTStopDayOfMonth(1u);
    RTC_WriteDSTStopMonth(1u);
    RTC_WriteDSTOffset(123u);

    /* Start */
    RTC_Start();

    /* Start LCD */
    LCD_Start();

    /* Prepare 0th column */
    LCD_Position(0u, 0u);
    LCD_PrintString("RTC    :  :  ");

    /* Print current time */
    tmpVar = RTC_ReadSecond();
    PrintDecNumber(tmpVar, 0u, 11u);
    tmpVar = RTC_ReadMinute();
    PrintDecNumber(tmpVar, 0u, 8u);
    tmpVar = RTC_ReadHour();
    PrintDecNumber(tmpVar, 0u, 5u);

    /* Prepare 1st column */
    LCD_Position(1u, 0u);
    LCD_PrintString("  -  - ");

    /* Get and print day of month */
    tmpVar = RTC_ReadDayOfMonth();
    PrintDecNumber(tmpVar, 1u, 0u);
    
    /* Get and print month */
    tmpVar = RTC_ReadMonth();
    PrintDecNumber(tmpVar, 1u, 3u);
    
    /* Get and print year */
    tmpVar = (uint8)(RTC_ReadYear() % 100u);
    PrintDecNumber(tmpVar, 1u, 6u);
    
    /* Get and print day of week */
    tmpVar = RTC_currentTimeDate.DayOfWeek;
    LCD_Position(1u, 9u);
    LCD_PutChar(tmpVar + 0x30u);

    /* Get status */
    tmpVar = RTC_ReadStatus();
    
    /* Get and print if year is leap */
    if(RTC_STATUS_LY & tmpVar)
    {
        LCD_Position(1u, 11u);
        LCD_PutChar('L');
    }
    else
    {
        LCD_Position(1u, 11u);
        LCD_PutChar(' ');
    }

    /* Get and print daytime AM/PM */
    if(RTC_STATUS_AM_PM & tmpVar)
    {
        LCD_Position(0u, 14u);
        LCD_PrintString("PM");
    }
    else
    {
        LCD_Position(0u, 14u);
        LCD_PrintString("AM");
    }

    /* Get and print DST status */
    if (RTC_STATUS_DST & tmpVar)
    {
        LCD_Position(1u, 13u);
        LCD_PutChar('D');
    }
    else
    {
        LCD_Position(1u, 13u);
        LCD_PutChar(' ');
    }

    /* Get and print alarm status */
    if (RTC_STATUS_AA & tmpVar)
    {
        LCD_Position(1u, 15u);
        LCD_PutChar('A');
    }
    else
    {
        LCD_Position(1u, 15u);
        LCD_PutChar(' ');
    }

    while(1u)
    {
        /* Make a 100 ms delay */
        CyDelay(100);
       
    }
}


/* [] END OF FILE */
